/*jslint nomen: true, indent: 2, maxerr: 3, maxlen: 80*/
/*global window, RSVP, rJS, jIO*/
(function (window, RSVP, rJS, jIO) {
 "use strict";
 rJS(window)

 .ready(function(){
   
   this.index = elasticlunr(function () {
    this.use(elasticlunr.fr);
    this.addField('title');
    this.addField('body');
    this.addField('link');
    this.setRef('id'); //id = x."/a/b" where x = substorage id, "/a/b" = item id in x  
  });

   this.feeds = jIO.createJIO({
      type : "indexeddb",
      database : "mynij-v1.2"
   });
   
   this.index_storage = jIO.createJIO({
     type : "indexeddb",
     database : "mynij-index"
   });

  this.sub_storages = [];

  this.info = jIO.createJIO({
    type : "indexeddb",
    database : "mynij-v1-info"
  });

  return this.feeds.put("feeds_doc", {});
 })
 
 .declareMethod ("add_attachment", function (title, rss){
   var gadget = this, sub_id;
    return gadget.feeds.putAttachment("feeds_doc", title, new Blob([rss], {type : "text/xml"}))
      .push(function (){
        var new_sub_storage = {
              type : "my_parser",
              document_id : "feeds_doc",
              attachment_id : title,
              parser : "rss",
              sub_storage : {
                type : "indexeddb",
                database : "mynij-v1.2"
              }
            };
        var tmp = jIO.createJIO(new_sub_storage);
        gadget.sub_storages.push(tmp);
        sub_id = gadget.sub_storages.length-1;
        return gadget.sub_storages[sub_id].allDocs();
      })
      .push(function(all_new_items){
        var i, promise_list = [];
        for (i = 0; i<all_new_items.data.rows.length; i+=1){
          promise_list.push(gadget.sub_storages[sub_id].get(all_new_items.data.rows[i].id));
        }
        return RSVP.all(promise_list);
      })
      .push(function(items){
        var i, id, doc;
        for (i=1; i<items.length; i++){
          id = sub_id+"./0/"+i;
          doc = {
            "id": id,
            "title": items[i].title,
            "body": items[i].description,
            "link": items[i].link
          };
          gadget.index.addDoc(doc);
        }
        return gadget.index_storage.put("index", {index : JSON.stringify(gadget.index)});
      });
 })
 
 .declareMethod("load_index", function(){
    var raw_index,
        gadget = this;
    return gadget.index_storage.get("index")
      .push(function(result){
        var key,
            doc;
        raw_index = JSON.parse(result.index);
        for (key in raw_index.documentStore.docs){
          doc = {
            "id" : raw_index.documentStore.docs[key].id,
            "title" : raw_index.documentStore.docs[key].title,
            "body" : raw_index.documentStore.docs[key].body,
            "link" : raw_index.documentStore.docs[key].link
          };
          gadget.index.addDoc(doc);
        }
      });
 })
 
 .declareMethod("allDocs", function (){
    var promise_list = [],
        i;
    for (i=0; i<this.sub_storages.length; i+=1){
      promise_list.push(this.sub_storages[i].allDocs());
    }
    return RSVP.all(promise_list);
 })

 .declareMethod("get", function (id_storage, id_item) {
    return this.sub_storages[id_storage].get(id_item);
 })

 .declareMethod("search", function() {
   return this.index.search(arguments[0]);
 })

 .declareMethod("loaded_doc", function(){
   return this.info.put.apply(this.info, arguments);
 })

 .declareMethod("all_loaded_docs", function(){
   return this.info.allDocs.apply(this.info, arguments);
 });
}(window, RSVP, rJS, jIO));
