(function (jIO, DOMParser, Node) {
  "use strict";
  
   /////////////////////////////////////////////////////////////
  // RSS Parser
  /////////////////////////////////////////////////////////////
  function RSSParser(txt) {
    this._dom_parser = new DOMParser().parseFromString(txt, 'text/xml');
  }
  RSSParser.prototype.parseElement = function (element) {
    var tag_element,
      i,
      j,
      attribute,
      result = {};
    for (i = element.childNodes.length - 1; i >= 0; i -= 1) {
      tag_element = element.childNodes[i];
      if ((tag_element.nodeType === Node.ELEMENT_NODE) &&
          (tag_element.tagName !== 'item')) {
        result[tag_element.tagName] = tag_element.textContent;
        for (j = tag_element.attributes.length - 1; j >= 0; j -= 1) {
          attribute = tag_element.attributes[j];
          if (attribute.value) {
            result[tag_element.tagName + '_' + attribute.name] =
              attribute.value;
          }
        }
      }
    }
    return result;
  };
  RSSParser.prototype.getDocumentList = function (include, id) {
    var result_list,
      item_list = this._dom_parser.querySelectorAll("rss > channel > item"),
      i;
      //console.log(item_list);
    if ((id === '/0') || (id === undefined)) {
      result_list = [{
        id: '/0',
        value: {}
      }];
      if (include) {
        result_list[0].doc = this.parseElement(
          this._dom_parser.querySelector("rss > channel")
        );
      }
    } else {
      result_list = [];
    }
    for (i = 0; i < item_list.length; i += 1) {
      if ((id === '/0/' + i) || (id === undefined)) {
        result_list.push({
          id: '/0/' + i,
          value: {}
        });
        if (include) {
          result_list[result_list.length - 1].doc =
            this.parseElement(item_list[i]);
        }
      }
    }
    return result_list;
  };
  
  /////////////////////////////////////////////////////////////
  // Helpers
  /////////////////////////////////////////////////////////////
  
  
  function getParser(storage) {
    return storage._sub_storage.getAttachment(storage._document_id,
                                              storage._attachment_id,
                                              {format: 'text'})
      .push(function (txt) {
        return new RSSParser(txt);
      });
  }
  
    /////////////////////////////////////////////////////////////
  // Storage
  /////////////////////////////////////////////////////////////
  function MyParserStorage(spec) {
    this._attachment_id = spec.attachment_id;
    this._document_id = spec.document_id;
    this._parser_name = spec.parser;
    this._sub_storage = jIO.createJIO(spec.sub_storage);
  }
  MyParserStorage.prototype.hasCapacity = function (capacity) {
    return (capacity === "list") || (capacity === 'include');
  };
  MyParserStorage.prototype.buildQuery = function (options) {
    if (options === undefined) {
      options = {};
    }
    return getParser(this)
      .push(function (parser) {
        return parser.getDocumentList((options.include_docs || false));
      });
  };
  MyParserStorage.prototype.get = function (id) {
    return getParser(this)
      .push(function (parser) {
        var result_list = parser.getDocumentList(true, id);
        //console.log(result_list);
        if (result_list.length) {
          return result_list[0].doc;
        }
        throw new jIO.util.jIOError(
          "Cannot find parsed document: " + id,
          404
        );
      });
  };
  MyParserStorage.prototype.put = function(id, metadata){
    return this._sub_storage.put(id, metadata);
  };
  MyParserStorage.prototype.putAttachment = function(id, name, blob){
    return this._sub_storage.putAttachment(id, name, blob);
  };
  jIO.addStorage('my_parser', MyParserStorage);

  
}(jIO, DOMParser, Node));